import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/decorator';
import { BaseController, FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import { PatientService } from '../service/patient.service';
import { CDTO_patient, CVO_patient, FVO_patient, QDTO_patient, QVO_patient, UDTO_patient, UVAO_patient } from '../interface/patient.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { get } from 'lodash';
import { verifyToken } from '../decorator/verifyToken.decorator';
@ApiTags(['病人'])
@Controller('/patient')
export class PatientController extends BaseController {
  @Inject()
  service: PatientService;

  /**
   * 患者扫描群组二维码,创建自己的数据并加入群组
   * @param data 患者数据
   * @param groupId 群组id
   */
  @Post('/inviteCreate/:groupId')
  @verifyToken()
  async inviteCreate(@Body() data: CDTO_patient, @Param('groupId') groupId: string) {
    if (!data.openid) throw new ServiceError('缺少微信关联', FrameworkErrorEnum.BAD_BODY);
    const dbData = await this.service.create(data);
    try {
      await this.service.joinGroup(get(dbData, '_id'), groupId);
      return dbData;
    } catch (error) {
      await this.service.delete(get(dbData, '_id'));
      throw error;
    }
  }

  @Post('/')
  @verifyToken()
  @Validate()
  @ApiResponse({ type: CVO_patient })
  async create(@Body() data: CDTO_patient) {
    const dbData = await this.service.create(data);
    const result = new CVO_patient(dbData);
    return result;
  }
  @Get('/')
  @verifyToken()
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_patient })
  async query(@Query() filter: QDTO_patient, @Query('skip') skip: number, @Query('limit') limit: number) {
    const list = await this.service.query(filter, { skip, limit });
    const data = [];
    for (const i of list) {
      const newData = new QVO_patient(i);
      data.push(newData);
    }
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/:id')
  @verifyToken()
  @ApiResponse({ type: FVO_patient })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FVO_patient(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @verifyToken()
  @ApiResponse({ type: UVAO_patient })
  async update(@Param('id') id: string, @Body() body: UDTO_patient) {
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @verifyToken()
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
